
#include <Arduino_AdvancedAnalog.h>

#define BUFSIZE 8 // tampon pour l'ADC
#define NBUF 4
uint32_t sample_rate = 10000;
AdvancedADC adc1(A0);

#define PERIODBUFSIZE 100 // tampon pour les périodes
uint32_t period[2][PERIODBUFSIZE];
uint32_t period_index;
uint8_t period_nbuf;


// transformation nombre -> tension
float offset = 2048;
float num2volt = 3.217/4096;

int8_t signe;
uint8_t derive = 0;
float epsilon = 1e-3;
uint32_t sample_count;
float last_sample;

#define BSIZE 41
#define ASIZE 1
#define WSIZE 41 // taille maximale de a et b
float b[41] = {-0,-6.05231e-05,-0.00041093,-0.000968941,-0.00111489,1.13996e-18,0.00275013,0.0063475,0.00870225,0.00716282,-3.89207e-18,-0.0120015,-0.0247287,-0.0313913,-0.0247172,6.64417e-18,0.0422416,0.0952615,0.14743,0.185655,0.199687,0.185655,0.14743,0.0952615,0.0422416,6.64417e-18,-0.0247172,-0.0313913,-0.0247287,-0.0120015,-3.89207e-18,0.00716282,0.00870225,0.0063475,0.00275013,1.13996e-18,-0.00111489,-0.000968941,-0.00041093,-6.05231e-05,-0};
float a[1] = {1.0};

float w[WSIZE];
uint16_t iw; // indice de wn

// transferts des données
#define GET_DATA 10
#define SET_DATA 11
#define DATA_0_SIZE PERIODBUFSIZE*4 // périodes
#define DATA_1_SIZE 4 // uint32_t, sample_rate
#define DATA_2_SIZE 1 // uint8_t, derive
#define DATA_3_SIZE 4 // float, epsilon
uint8_t data_1[DATA_1_SIZE];
uint8_t data_2[DATA_2_SIZE];
uint8_t data_3[DATA_3_SIZE];
bool data_0_ready = false;
bool data_0_request = false;



void get_data() {
  char n;
  while (Serial.available()<1) {};
  n = Serial.read();
  if (n==0) data_0_request = true;
}

void send_data() {
  if ((data_0_ready)&&(data_0_request)) {
    
      data_0_ready = false;
      data_0_request = false;
      uint8_t nbuf;
      if (period_nbuf==0) nbuf=1; else nbuf=0;
      Serial.write((uint8_t*)(period[nbuf]),DATA_0_SIZE);
  }
  
}

void set_data() {
  char n;
  while (Serial.available()<1) {};
  n = Serial.read();
  
  if (n==1) {
    while (Serial.available()<DATA_1_SIZE) {};
    Serial.readBytes(data_1,DATA_1_SIZE);
    memcpy(&sample_rate,data_1,DATA_1_SIZE);
    adc1.stop();
    adc1.begin(AN_RESOLUTION_12, sample_rate, BUFSIZE, NBUF);
  }
  else if (n==2) {
    while (Serial.available()<DATA_2_SIZE) {};
    Serial.readBytes(data_2,DATA_2_SIZE);
    memcpy(&derive,data_2,DATA_2_SIZE);
  }
  else if (n==3) {
    while (Serial.available()<DATA_3_SIZE) {};
    Serial.readBytes(data_3,DATA_3_SIZE);
    memcpy(&epsilon,data_3,DATA_3_SIZE);
  }
  
}

void read_serial() {
   char com;
   if (Serial.available()>0) {
        com = Serial.read();
        if (com==GET_DATA) get_data();
        else if (com==SET_DATA) set_data();
   }
}	

void setup() {
  pinMode(13,OUTPUT);
  Serial.begin(500000);
  while(!Serial);
  for (int i=0; i<WSIZE; i++) w[i] = 0.0f;
  iw = 0;
  last_sample = 0;
  signe = 2;
  sample_count = 0;
  period_index = 0;
  period_nbuf = 0;
  if (!adc1.begin(AN_RESOLUTION_12, sample_rate, BUFSIZE, NBUF)) {
        Serial.println("Impossible d'ouvrir ADC");
        while (1);
    }
}

void analyse() {
  GPIOH->BSRR |= GPIO_BSRR_BS6; //digitalWrite(13,HIGH);
  SampleBuffer buf1 = adc1.read();
  uint16_t *buf1_data = buf1.data();
  int16_t jw;
  float y;
  float ytemp;
  for (int i=0; i<buf1.size(); i++) {
          w[iw] = 1.0/a[0]*(buf1_data[i]-offset)*num2volt;
          jw = iw-1;
          if (jw==-1) jw=WSIZE-1;
          for (uint16_t k=1; k<ASIZE; k++) {
              w[iw] -=  a[k]*w[jw];
              jw = iw-1;
              if (jw==-1) jw=WSIZE-1;
          }
          jw = iw;
          y = 0.0f;
          for (uint16_t k=0; k<BSIZE; k++) {
              y += b[k]*w[jw];
              jw--;
              if (jw==-1) jw=WSIZE-1;
          }
          iw++;
          if (iw==WSIZE) iw=0;
          if (derive) {
            ytemp = y;
            y = (y-last_sample)*sample_rate; // dérivée
            last_sample = ytemp;
          }
          sample_count++;
          if (signe==2) { // initialisation
            if (y>epsilon) signe=1; 
            else if (y<-epsilon) signe=-1;
          }
          else {
            if ((signe>0)&&(y<-epsilon)) signe=-1;
            else if ((signe<0)&&(y>epsilon)) { // passage par zéro par valeur croissante
              signe=1;
              period[period_nbuf][period_index] = sample_count; // sauvegarde dans le tampon
              sample_count = 0;
              period_index++;
              if (period_index==PERIODBUFSIZE) {
                period_index = 0;
                if (period_nbuf==0) period_nbuf=1; else period_nbuf = 0;
                data_0_ready = true;
              }
            }
          }
        }
    buf1.release();
    GPIOH->BSRR |= GPIO_BSRR_BR6; //digitalWrite(13,LOW);
}

void loop() {
  if (adc1.available()) analyse();
  read_serial();
  send_data();
}
       
                